@@ -111,8 +111,8 @@ fn security_context_color() {
 
     let scc_clone = super::SecurityContextColors::clone(&scc);
     assert_eq!(scc, scc_clone);
-    assert!(!(scc < scc_clone));
-    assert!(!(scc > scc_clone));
+    assert!(scc >= scc_clone);
+    assert!(scc <= scc_clone);
     assert_ne!(scc, super::SecurityContextColors::default());
     let _ignored = format!("{:?}", &scc);
     let mut ht = HashSet::new();
@@ -534,8 +534,8 @@ fn rgb() {
     let rgb = super::RGB::new(0x22, 0, 0);
     let rgb_clone = super::RGB::clone(&rgb);
     assert_eq!(rgb, rgb_clone);
-    assert!(!(rgb < rgb_clone));
-    assert!(!(rgb > rgb_clone));
+    assert!(rgb >= rgb_clone);
+    assert!(rgb <= rgb_clone);
     assert_ne!(rgb, super::RGB::default());
     let _ignored = format!("{:?}", &rgb);
     let mut ht = HashSet::new();
@@ -547,8 +547,8 @@ fn layer_colors() {
     let lc = super::LayerColors::new(super::RGB::new(0x22, 0, 0), super::RGB::new(0x11, 0, 0));
     let lc_clone = super::LayerColors::clone(&lc);
     assert_eq!(lc, lc_clone);
-    assert!(!(lc < lc_clone));
-    assert!(!(lc > lc_clone));
+    assert!(lc >= lc_clone);
+    assert!(lc <= lc_clone);
     assert_ne!(lc, super::LayerColors::default());
     let _ignored = format!("{:?}", &lc);
     let mut ht = HashSet::new();
@@ -594,7 +594,7 @@ fn security_class_new() {
 
     let _name = sc.full_access_vector_name(0).unwrap();
     let _name = sc.full_access_vector_name(1).unwrap();
-    sc.full_access_vector_name(u32::MAX).unwrap_err();
+    let _name = sc.full_access_vector_name(u32::MAX).unwrap();
 
     sc.access_vector_bit("invalid").unwrap_err();
 
@@ -638,7 +638,7 @@ fn opaque_security_context() {
             assert!(expected_range.is_none());
         }
 
-        osc.set_user("user2").unwrap();
+        osc.set_user_str("user2").unwrap();
 
         assert_eq!(osc.user().unwrap().to_str().ok(), Some("user2"));
         assert_eq!(osc.role().unwrap().to_str().ok(), Some("role1"));
@@ -649,7 +649,7 @@ fn opaque_security_context() {
             assert!(expected_range.is_none());
         }
 
-        osc.set_role("role2").unwrap();
+        osc.set_role_str("role2").unwrap();
 
         assert_eq!(osc.user().unwrap().to_str().ok(), Some("user2"));
         assert_eq!(osc.role().unwrap().to_str().ok(), Some("role2"));
@@ -660,7 +660,7 @@ fn opaque_security_context() {
             assert!(expected_range.is_none());
         }
 
-        osc.set_type("type2").unwrap();
+        osc.set_type_str("type2").unwrap();
 
         assert_eq!(osc.user().unwrap().to_str().ok(), Some("user2"));
         assert_eq!(osc.role().unwrap().to_str().ok(), Some("role2"));
@@ -671,7 +671,7 @@ fn opaque_security_context() {
             assert!(expected_range.is_none());
         }
 
-        osc.set_range("range2").unwrap();
+        osc.set_range_str("range2").unwrap();
 
         assert_eq!(osc.user().unwrap().to_str().ok(), Some("user2"));
         assert_eq!(osc.role().unwrap().to_str().ok(), Some("role2"));
@@ -685,7 +685,7 @@ fn opaque_security_context() {
 #[test]
 fn kernel_support() {
     let r = super::kernel_support();
-    let _ignored = format!("{:?}", r);
+    let _ignored = format!("{r:?}");
 }
 
 #[test]
@@ -703,7 +703,7 @@ fn boot_mode() {
 #[test]
 fn current_mode() {
     let r = super::current_mode();
-    let _ignored = format!("{:?}", r);
+    let _ignored = format!("{r:?}");
 }
 
 #[test]
@@ -827,6 +827,6 @@ fn default_type_for_role() {
 
 #[test]
 fn set_dynamic_mapping() {
-    let _type = super::set_dynamic_mapping(&[] as &[(&str, &[&str])]).unwrap();
-    let _type = super::set_dynamic_mapping(&[("file", &["read", "write"] as &[&str])]).unwrap();
+    super::set_dynamic_mapping(&[] as &[(&str, &[&str])]).unwrap();
+    super::set_dynamic_mapping(&[("file", &["read", "write"] as &[&str])]).unwrap();
 }
